@@ -26,7 +26,7 @@ from fakeredis._helpers import (
     null_terminate,
     Database,
 )
-from fakeredis._zset import ZSet
+from fakeredis.model import ZSet, ExpiringMembersSet
 
 SORTED_SET_METHODS = {
     "ZUNIONSTORE": lambda s1, s2: s1 | s2,
@@ -49,27 +49,20 @@ class SortedSetCommandsMixin:
         self._db: Database
         # Sorted set commands
 
-    def _zpop(
-            self, key: CommandItem, count: int, reverse: bool, flatten_list: bool
-    ) -> List[List[Any]]:
+    def _zpop(self, key: CommandItem, count: int, reverse: bool, flatten_list: bool) -> List[List[Any]]:
         zset = key.value
         members = list(zset)
         if reverse:
             members.reverse()
         members = members[:count]
-        res = [
-            [bytes(member), self._encodefloat(zset.get(member), True)]
-            for member in members
-        ]
+        res = [[bytes(member), self._encodefloat(zset.get(member), True)] for member in members]
         if flatten_list:
             res = list(itertools.chain.from_iterable(res))
         for item in members:
             zset.discard(item)
         return res
 
-    def _bzpop(
-            self, keys: List[bytes], reverse: bool, first_pass: bool
-    ) -> Optional[List[Union[bytes, List[bytes]]]]:
+    def _bzpop(self, keys: List[bytes], reverse: bool, first_pass: bool) -> Optional[List[Union[bytes, List[bytes]]]]:
         for key in keys:
             item = CommandItem(key, self._db, item=self._db.get(key), default=[])
             temp_res = self._zpop(item, 1, reverse, True)
@@ -150,9 +143,7 @@ class SortedSetCommandsMixin:
         items = [
             (
                 (
-                    0.0 + Float.decode(elements[j])
-                    if self.version >= (7,)
-                    else Float.decode(elements[j]),
+                    0.0 + Float.decode(elements[j]) if self.version >= (7,) else Float.decode(elements[j]),
                     elements[j + 1],
                 )
             )
@@ -172,18 +163,10 @@ class SortedSetCommandsMixin:
             update = update or (count == 1 and nx and item_name not in zset)
             update = update or (count == 1 and xx and item_name in zset)
             update = update or (
-                    gt
-                    and (
-                            (item_name in zset and zset.get(item_name) < item_score)
-                            or (not xx and item_name not in zset)
-                    )
+                gt and ((item_name in zset and zset.get(item_name) < item_score) or (not xx and item_name not in zset))
             )
             update = update or (
-                    lt
-                    and (
-                            (item_name in zset and zset.get(item_name) > item_score)
-                            or (not xx and item_name not in zset)
-                    )
+                lt and ((item_name in zset and zset.get(item_name) > item_score) or (not xx and item_name not in zset))
             )
 
             if update:
@@ -224,19 +207,13 @@ class SortedSetCommandsMixin:
 
     @command((Key(ZSet), StringTest, StringTest))
     def zlexcount(self, key, _min, _max):
-        return key.value.zlexcount(
-            _min.value, _min.exclusive, _max.value, _max.exclusive
-        )
+        return key.value.zlexcount(_min.value, _min.exclusive, _max.value, _max.exclusive)
 
-    def _zrangebyscore(
-            self, key, _min, _max, reverse, withscores, offset, count
-    ) -> List[bytes]:
+    def _zrangebyscore(self, key, _min, _max, reverse, withscores, offset, count) -> List[bytes]:
         zset = key.value
         if reverse:
             _min, _max = _max, _min
-        items = list(
-            zset.irange_score(_min.lower_bound, _max.upper_bound, reverse=reverse)
-        )
+        items = list(zset.irange_score(_min.lower_bound, _max.upper_bound, reverse=reverse))
         items = self._limit_items(items, offset, count)
         items = self._apply_withscores(items, withscores)
         return items
@@ -244,9 +221,7 @@ class SortedSetCommandsMixin:
     def _zrange(self, key, start, stop, reverse, withscores, byscore) -> List[bytes]:
         zset = key.value
         if byscore:
-            items = zset.irange_score(
-                start.lower_bound, stop.upper_bound, reverse=reverse
-            )
+            items = zset.irange_score(start.lower_bound, stop.upper_bound, reverse=reverse)
         else:
             start, stop = Int.decode(start.bytes_val), Int.decode(stop.bytes_val)
             start, stop = fix_range(start, stop, len(zset))
@@ -357,17 +332,26 @@ class SortedSetCommandsMixin:
         count = -1 if count is None else count
         return self._zrangebyscore(key, _min, _max, True, withscores, offset, count)
 
-    @command((Key(ZSet), bytes))
-    def zrank(self, key, member):
+    @command(name="ZRANK", fixed=(Key(ZSet), bytes), repeat=(bytes,))
+    def zrank(self, key: CommandItem, member: bytes, *args: bytes) -> Union[None, int, List[Union[int, bytes]]]:
+        (withscore,), _ = extract_args(args, ("withscore",))
         try:
-            return key.value.rank(member)
+            rank, score = key.value.rank(member)
+            if withscore:
+                return [rank, self._encodefloat(score, False)]
+            return rank
         except KeyError:
             return None
 
-    @command((Key(ZSet), bytes))
-    def zrevrank(self, key, member):
+    @command(name="ZREVRANK", fixed=(Key(ZSet), bytes), repeat=(bytes,))
+    def zrevrank(self, key: CommandItem, member: bytes, *args: bytes) -> Union[None, int, List[Union[int, bytes]]]:
+        (withscore,), _ = extract_args(args, ("withscore",))
         try:
-            return len(key.value) - 1 - key.value.rank(member)
+            rank, score = key.value.rank(member)
+            rev_rank = len(key.value) - 1 - rank
+            if withscore:
+                return [rev_rank, self._encodefloat(score, False)]
+            return rev_rank
         except KeyError:
             return None
 
@@ -383,9 +367,7 @@ class SortedSetCommandsMixin:
 
     @command((Key(ZSet), StringTest, StringTest))
     def zremrangebylex(self, key, _min, _max):
-        items = key.value.irange_lex(
-            _min.value, _max.value, inclusive=(not _min.exclusive, not _max.exclusive)
-        )
+        items = key.value.irange_lex(_min.value, _max.value, inclusive=(not _min.exclusive, not _max.exclusive))
         return self.zrem(key, *items)
 
     @command((Key(ZSet), ScoreTest, ScoreTest))
@@ -418,7 +400,7 @@ class SortedSetCommandsMixin:
 
     @staticmethod
     def _get_zset(value):
-        if isinstance(value, set):
+        if isinstance(value, ExpiringMembersSet):
             zset = ZSet()
             for item in value:
                 zset[item] = 1.0
@@ -440,10 +422,7 @@ class SortedSetCommandsMixin:
         while i < len(args):
             arg = args[i]
             if casematch(arg, b"weights") and i + numkeys < len(args):
-                weights = [
-                    Float.decode(x, decode_error=msgs.INVALID_WEIGHT_MSG)
-                    for x in args[i + 1: i + numkeys + 1]
-                ]
+                weights = [Float.decode(x, decode_error=msgs.INVALID_WEIGHT_MSG) for x in args[i + 1 : i + numkeys + 1]]
                 i += numkeys + 1
             elif casematch(arg, b"aggregate") and i + 1 < len(args):
                 aggregate = null_terminate(args[i + 1])
@@ -455,9 +434,7 @@ class SortedSetCommandsMixin:
 
         sets = []
         for i in range(numkeys):
-            item = CommandItem(
-                args[i], self._db, item=self._db.get(args[i]), default=ZSet()
-            )
+            item = CommandItem(args[i], self._db, item=self._db.get(args[i]), default=ZSet())
             sets.append(self._get_zset(item.value))
 
         out_members = set(sets[0])
@@ -521,8 +498,8 @@ class SortedSetCommandsMixin:
 
     @command(
         (
-                Int,
-                bytes,
+            Int,
+            bytes,
         ),
         (bytes,),
     )
@@ -539,8 +516,8 @@ class SortedSetCommandsMixin:
 
     @command(
         (
-                Int,
-                bytes,
+            Int,
+            bytes,
         ),
         (bytes,),
     )
@@ -557,8 +534,8 @@ class SortedSetCommandsMixin:
 
     @command(
         (
-                Int,
-                bytes,
+            Int,
+            bytes,
         ),
         (bytes,),
     )
@@ -576,8 +553,8 @@ class SortedSetCommandsMixin:
     @command(
         name="ZINTERCARD",
         fixed=(
-                Int,
-                bytes,
+            Int,
+            bytes,
         ),
         repeat=(bytes,),
     )
@@ -593,9 +570,7 @@ class SortedSetCommandsMixin:
         return min(limit, len(res))
 
     @command(name="ZMSCORE", fixed=(Key(ZSet), bytes), repeat=(bytes,))
-    def zmscore(
-            self, key: CommandItem, *members: Union[str, bytes]
-    ) -> list[Optional[float]]:
+    def zmscore(self, key: CommandItem, *members: Union[str, bytes]) -> list[Optional[float]]:
         """Get the scores associated with the specified members in the sorted set
         stored at the key.
 
@@ -603,9 +578,7 @@ class SortedSetCommandsMixin:
         is returned.
         """
         scores = map(
-            lambda score: score
-            if score is None
-            else self._encodefloat(score, humanfriendly=False),
+            lambda score: score if score is None else self._encodefloat(score, humanfriendly=False),
             map(key.value.get, members),
         )
         return list(scores)
@@ -652,17 +625,15 @@ class SortedSetCommandsMixin:
             args = args[:-2]
         else:
             count = 1
-        if len(args) != numkeys + 1 or (
-                not casematch(args[-1], b"min") and not casematch(args[-1], b"max")
-        ):
+        if len(args) != numkeys + 1 or (not casematch(args[-1], b"min") and not casematch(args[-1], b"max")):
             raise SimpleError(msgs.SYNTAX_ERROR_MSG)
 
         return self._zmpop(args[:-1], count, casematch(args[-1], b"max"), False)
 
     @command(
         fixed=(
-                Timeout,
-                Int,
+            Timeout,
+            Int,
         ),
         repeat=(bytes,),
     )
@@ -674,14 +645,10 @@ class SortedSetCommandsMixin:
             args = args[:-2]
         else:
             count = 1
-        if len(args) != numkeys + 1 or (
-                not casematch(args[-1], b"min") and not casematch(args[-1], b"max")
-        ):
+        if len(args) != numkeys + 1 or (not casematch(args[-1], b"min") and not casematch(args[-1], b"max")):
             raise SimpleError(msgs.SYNTAX_ERROR_MSG)
 
         return self._blocking(
             timeout,
-            functools.partial(
-                self._zmpop, args[:-1], count, casematch(args[-1], b"max")
-            ),
+            functools.partial(self._zmpop, args[:-1], count, casematch(args[-1], b"max")),
         )
